<?php

require_once ABSPATH . 'wp-admin/includes/upgrade.php';
require_once __DIR__ . '/admin/MnetAdTag.php';
require_once __DIR__ . '/admin/MnetPluginUtils.php';
require_once __DIR__ . '/utils/MnetAdSlot.php';
require_once __DIR__ . '/utils/constants.php';
class MnetDbManager
{
    public static $MNET_DB_VERSION = 2.4;
    public static $WP_OPTION_MNET_DB_VERSION = 'MNET_DB_VERSION';

    public static $MNET_AD_TAGS = 'ad_tags';
    public static $MNET_AD_SLOTS = 'ad_slots';
    public static $MNET_AD_PARAGRAPH_MAPPING = 'ad_paragraph_mapping';
    public static $MNET_AD_POST_MAPPING = 'ad_post_mapping';
    public static $MNET_LOG_RETRY = 'log_retry';
    public static $MNET_BLOCKED_URLS = 'blocked_urls';
    public static $MNET_SLOT_BLOCKED_URLS = 'slot_blocked_urls';
    public static $MNET_EXTERNAL_ADS = 'external_ad';

    public static $schema = array(
        'ad_tags' => '
            id mediumint(9) AUTO_INCREMENT,
            updated_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            created_at TIMESTAMP NOT NULL,
            type varchar(28) NOT NULL,
            ad_tag_id bigint(56) NOT NULL,
            name varchar(128) NOT NULL,
            width smallint(4),
            height smallint(4),
            product_type_id int(8),
            crid bigint(56),
            ad_code TEXT DEFAULT "",
            version_id mediumint(28),
            status varchar(10),
            primary key  (id)
        ',
        'ad_slots' => "
            id mediumint(9) AUTO_INCREMENT,
            updated_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            created_at TIMESTAMP NOT NULL,
            ptype_id int(8),
            page varchar(128) NOT NULL,
            tag_id mediumint(9),
            position varchar(128) NOT NULL,
            ad_type TINYINT(1) DEFAULT 1,
            debug TINYINT(1) DEFAULT 0,
            options LONGTEXT NOT NULL,
            custom_css LONGTEXT DEFAULT '',
            primary key  (id)
        ",
        'ad_paragraph_mapping' => "
            id mediumint(9) AUTO_INCREMENT,
            ad_slot_id mediumint(9),
            paragraph_no int(8),
            primary key  (id)
        ",
        'ad_post_mapping' => "
            id mediumint(9) AUTO_INCREMENT,
            ad_slot_id mediumint(9),
            post_no int(8),
            primary key  (id)
        ",
        'log_retry' => '
            id mediumint(9) AUTO_INCREMENT,
            updated_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            created_at TIMESTAMP NOT NULL,
            content TEXT NOT NULL,
            primary key  (id)
        ',
        'blocked_urls' => '
            id mediumint(9) AUTO_INCREMENT,
            url varchar(256) NOT NULL,
            primary key  (id)
        ',
        'slot_blocked_urls' => "
            id mediumint(9) AUTO_INCREMENT,
            slot_name varchar(256) NOT NULL,
            url_list TEXT DEFAULT '',
            primary key  (id)
        ",
        'external_ad' => "
            id mediumint(9) AUTO_INCREMENT,
            slot_id mediumint(9),
            code TEXT NOT NULL,
            meta TEXT DEFAULT '',
            primary key  (id)
        "
    );

    public static function createTablesOnActivate()
    {
        $stored_version = get_option(self::$WP_OPTION_MNET_DB_VERSION);

        if (!$stored_version || $stored_version < self::$MNET_DB_VERSION) {
            self::createUpdateDatabase();
            MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_DB_VERSION, self::$MNET_DB_VERSION);
            MnetLogManager::logEvent('Activate');
        }
    }

    public static function createUpdateDatabase()
    {
        global $wpdb;
        $errors = array();
        $charset_collate = $wpdb->get_charset_collate();
        $custom_charset_collate = "DEFAULT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci";
        foreach (static::$schema as $name => $columns) {
            $table_name = static::tableName($name);
            $error = self::createTable($table_name, $columns, $charset_collate);

            if (
                $charset_collate !== $custom_charset_collate
                && preg_match("/unknown character set/i", $error)
            ) {
                $error = '';
                $error = self::createTable($table_name, $columns, $custom_charset_collate);
            }

            if ($error !== "") {
                array_push($errors, array($table_name => $error));
            }
        }
        if (count($errors) > 0) {
            MnetAdHandleAjaxCalls::logAdminPageError(array('createTableErrors' => $errors));
        }
    }

    public static function createTable($tableName, $columns, $charset)
    {
        global $wpdb;
        $query = "CREATE TABLE {$tableName} ({$columns}) $charset;";
        dbDelta($query);
        return $wpdb->last_error;
    }

    public static function tableName($name)
    {
        global $wpdb;
        if (!isset(static::$schema[$name])) {
            throw new Exception("Unknown table {$name}");
        }
        return "{$wpdb->prefix}mnet_{$name}";
    }

    public static function getDbVersion()
    {
        global $wpdb;
        return $wpdb->db_version();
    }

    public static function all($table_name, $columns = "*")
    {
        global $wpdb;
        $query = "select $columns from " . static::tableName($table_name);
        return $wpdb->get_results($query, ARRAY_A);
    }

    public static function getDataById($table_name, $id, $id_name = 'id', $columns = "*")
    {
        global $wpdb;
        $query = "select $columns from " . static::tableName($table_name) . " where " . $id_name . " = " . intval($id);
        return $wpdb->get_results($query, ARRAY_A);
    }

    public static function getDataWithClauses($table_name, $clauses)
    {
        global $wpdb;
        $query = "Select * from " . static::tableName($table_name);
        $clause_str = '';
        foreach ($clauses as $key => $value) {
            $clause_str .= "{$key}=" . (is_string($value) ? "'{$value}'" : "$value") . " AND ";
        }
        rtrim($clause_str, ' AND ');
        if (strlen($clause_str)) {
            $query .= " where " . $clause_str;
        }
        return $wpdb->get_results($query, ARRAY_A);
    }

    public static function getAdTagBySize($sizes)
    {
        global $wpdb;
        $size_list = implode("','", $sizes);
        $size_set = implode(",", $sizes);

        $columns = MnetAdTag::$requiredColumns;

        $query = "Select $columns, concat(width,'x',height) as ad_size from " . static::tableName(self::$MNET_AD_TAGS) . " where concat(width,'x',height) in ('" . $size_list . "') ORDER BY find_in_set(ad_size, '$size_set')";
        $preferredSizeAdtags = $wpdb->get_results($query, ARRAY_A);
        $query = "Select $columns, concat(width,'x',height) as ad_size from " . static::tableName(self::$MNET_AD_TAGS) . " where concat(width,'x',height) not in ('" . $size_list . "')";
        $remainingSizeAdtags = $wpdb->get_results($query, ARRAY_A);
        return array_merge($preferredSizeAdtags, $remainingSizeAdtags);
    }

    public static function getAdSlotsByTagId($tag_ids = array())
    {
        global $wpdb;
        $columns = MnetAdSlot::$requiredColumns;
        $query = 'SELECT ' . $columns . ' from ' . static::tableName(self::$MNET_AD_SLOTS) . ' WHERE tag_id in (' . implode(',', $tag_ids) . ')';
        return $wpdb->get_results($query, ARRAY_A);
    }

    public static function getAdSlots($position = null, $page = null, $tag_types = array())
    {
        global $wpdb;
        $columns = MnetAdSlot::$requiredColumns;
        $query = "Select $columns from " . static::tableName(self::$MNET_AD_SLOTS);
        $positionClause = '';
        $pageClause = '';
        $tagTypeClause = '';
        if ($position) {
            if (!is_array($position)) {
                $position = array($position);
            }
            $positionClause = " where position IN ('" . implode("','", array_values(array_filter($position))) . "')";
        }
        if ($page) {
            $pageClause = (!empty($positionClause) ? " AND " : " where ") . "page='{$page}'";
        }
        if (!empty($tag_types)) {
            $tagTypeClause = (!empty($positionClause) || !empty($pageClause) ? " AND " : " where ");
            $tagTypeClause .= "ptype_id in (" . implode(',', $tag_types) . ")";
        }
        $query .= $positionClause . $pageClause . $tagTypeClause;
        $slots = $wpdb->get_results($query, ARRAY_A);
        return array_map(function ($slot) {
            $slot['external_code'] = '';
            if ($slot['ad_type'] == MNET_AD_TYPE_EXTERNAL) {
                $extCode = self::getExternalCode($slot['id']);
                if (!empty($extCode)) {
                    $external_code = $extCode[0];
                    $slot['external_code'] = html_entity_decode($external_code['code']);
                }
            }
            return $slot;
        }, $slots);
    }

    public static function getAdSlotsForPage($page, $tag_types = array())
    {
        return self::getAdSlots(null, $page, $tag_types);
    }

    public static function getAdSlot($slot_id)
    {
        return self::getDataById(self::$MNET_AD_SLOTS, $slot_id);
    }

    public static function getExternalCode($slot_id)
    {
        return self::getDataById(self::$MNET_EXTERNAL_ADS, $slot_id, 'slot_id');
    }

    public static function updateAdSlot($update_data, $slot_id)
    {
        self::updateData(self::$MNET_AD_SLOTS, $update_data, array('id' => $slot_id));
    }

    public static function insertData($table_name, $data, $has_create_timestamp = false)
    {
        global $wpdb;
        if ($has_create_timestamp) {
            $data['created_at'] = current_time('mysql');
        }
        $wpdb->insert(static::tableName($table_name), $data);
        return $wpdb->insert_id;
    }

    public static function updateData($table_name, $update_data, $target_row)
    {
        global $wpdb;
        return $wpdb->update(static::tableName($table_name), $update_data, $target_row);
    }

    public static function deleteData($table_name, $target_row)
    {
        global $wpdb;
        return $wpdb->delete(static::tableName($table_name), $target_row);
    }

    public static function getRowCount($table_name, $clause = array())
    {
        global $wpdb;
        $query = "SELECT count(*) as count from " . static::tableName($table_name);
        if (!empty($clause)) {
            $clauseStr = '';
            foreach ($clause as $key => $value) {
                if (!empty($clauseStr)) {
                    $clauseStr .= ' AND ';
                }
                $clauseStr .= $key . '=' . (is_string($value) ? '\'' . $value . '\'' : $value);
            }
            $query .= ' WHERE ' . $clauseStr;
        }
        $count = Arr::get($wpdb->get_results($query, ARRAY_A), '0.count', null);
        return !empty($count) ? $count : 0;
    }

    public static function saveAdTags($ad_tags)
    {
        global $wpdb;
        $table_name = static::tableName(self::$MNET_AD_TAGS);
        return array_map(function ($ad_tag) use ($wpdb, $table_name) {
            return $wpdb->insert($table_name, $ad_tag);
        }, $ad_tags);
    }

    public static function saveAdTag($ad_tag)
    {
        global $wpdb;
        $result = self::getDataById(self::$MNET_AD_TAGS, $ad_tag['ad_tag_id'], 'ad_tag_id', 'id');
        if ($wpdb->num_rows) {
            $ad_tag_id = intval($ad_tag['ad_tag_id']);
            unset($ad_tag['ad_tag_id']);
            self::updateData(self::$MNET_AD_TAGS, $ad_tag, array('id' => intval($result[0]['id']), 'ad_tag_id' => $ad_tag_id));
            return $result[0]['id'];
        }
        return self::insertData(self::$MNET_AD_TAGS, $ad_tag, true);
    }

    public static function removeAdSlot($slot_id)
    {
        $slot = self::getAdSlot($slot_id);
        if (!empty($slot)) {
            try {
                self::deleteData(self::$MNET_AD_SLOTS, array('id' => $slot_id));
                self::deleteData(self::$MNET_AD_PARAGRAPH_MAPPING, array('ad_slot_id' => $slot_id));
                self::deleteData(self::$MNET_AD_POST_MAPPING, array('ad_slot_id' => $slot_id));
                self::deleteData(self::$MNET_SLOT_BLOCKED_URLS, array('slot_name' => $slot[0]['page'] . '_' . $slot[0]['position']));
                self::deleteData(self::$MNET_EXTERNAL_ADS, array('slot_id' => $slot_id));
                return true;
            } catch (\Exception $e) { }
        }
        return false;
    }

    public static function getNumberForSlot($type, $slot_id)
    {
        global $wpdb;
        $results = $wpdb->get_results("Select " . $type . "_no from " . static::tableName('ad_' . $type . '_mapping') . " where ad_slot_id=" . $slot_id, ARRAY_A);
        if ($wpdb->num_rows == 0) {
            return false;
        }
        return $results[0][$type . '_no'];
    }

    public static function clearExpiredAdtagsFromDb($new_ad_tag_ids)
    {
        global $wpdb;
        $clause = count($new_ad_tag_ids) ? ' where ad_tag_id not in (' . implode(',', $new_ad_tag_ids) . ')' : '';
        $query = 'Select id from ' . static::tableName(self::$MNET_AD_TAGS) . $clause;
        $expired_ad_tags = $wpdb->get_results($query, ARRAY_A);
        foreach ($expired_ad_tags as $ad_tag) {
            $slot_ids = $wpdb->get_results('Select id from ' . static::tableName(self::$MNET_AD_SLOTS) . ' where tag_id =' . $ad_tag['id'], ARRAY_A);
            foreach ($slot_ids as $slot_id) {
                self::removeAdSlot($slot_id['id']);
            }
            $wpdb->delete(static::tableName(self::$MNET_AD_TAGS), array('id' => $ad_tag['id']));
        }
    }

    public static function insertLog($log)
    {
        global $wpdb;
        $wpdb->insert(
            static::tableName(self::$MNET_LOG_RETRY),
            array(
                'content' => json_encode($log),
                'created_at' => current_time('mysql')
            ),
            array(
                '%s',
            )
        );
        return $wpdb->insert_id;
    }

    public static function getLogs()
    {
        return self::all(self::$MNET_LOG_RETRY);
    }

    public static function deleteLog($log)
    {
        global $wpdb;
        return $wpdb->delete(static::tableName(self::$MNET_LOG_RETRY), array('id' => $log['id']));
    }

    public static function truncateIfExists($table_name)
    {
        global $wpdb;
        if ($wpdb->get_var("SHOW TABLES LIKE '" . static::tableName($table_name) . "'") == static::tableName($table_name)) {
            $wpdb->query("TRUNCATE TABLE " . static::tableName($table_name));
            return true;
        }
        return false;
    }

    public static function clearDatabase()
    {
        foreach (static::$schema as $key => $value) {
            self::truncateIfExists($key);
        }
    }

    public static function clearSlots()
    {
        $cleared = true;
        $tables = array(
            self::$MNET_AD_SLOTS,
            self::$MNET_SLOT_BLOCKED_URLS,
            self::$MNET_AD_PARAGRAPH_MAPPING,
            self::$MNET_AD_POST_MAPPING,
            self::$MNET_EXTERNAL_ADS
        );
        foreach ($tables as $table) {
            try {
                self::truncateIfExists($table);
            } catch (\Exception $e) {
                $cleared = false;
            }
        }
        return $cleared;
    }

    public static function getAdHeadCodes($ptypes = array())
    {
        if (empty($ptypes)) return array();
        $headCodes = json_decode(get_option(MnetAdTag::$AD_HEAD_CODE_KEY, "[]", true));
        $codes = array();
        foreach ($ptypes as $ptype) {
            $codes[] = Arr::get($headCodes, 'ptype_' . $ptype, '');
        }
        return $codes;
    }
}
